;##################################################################
;
;   TesbWan for TI-86 (Brick handling)
;
;   Programmed by Patrick Davidson (pad@calc.org)
;        
;   This program is in the public domain.  There is no warranty.
;
;   This file was last updated August 24, 2002.
;
;##################################################################

;############## Check for and handle collision of ball with a brick
;
; Input: A = Y coordinate, D = X coordinate
;
; This routine is called after the ball is moved.  It checks the map at
; the ball's position for a collision with a brick, and signals whether a
; brick was hit.  It also updates the map (and redraws the corresponding
; tile) if appropriate.  The coordinates given should be from the leading
; edge of the ball.
; 
; May change all registers.
;
; Return value: zero flag = no collision.


check_brick_hit:
        call    DIV_A_6
        ld      a,c
        ld      (temp_y),a
        add     a,a
        add     a,a
        add     a,c
        add     a,a
        add     a,a             ; A = 20 * Y coordinate
        ld      hl,working_map
        call    ADD_HL_A        ; HL -> start of this column in map
        ld      a,d
        ld      (temp_x),a
        rra
        rra
        rra
        and     31
        call    ADD_HL_A        ; HL -> this brick in map
        ld      a,(hl)
        or      a
        ret     z
        dec     a
        jr      z,hit_one
        dec     a
        jr      z,hit_two
        ret

hit_one:
        dec     (hl)
        ld      hl,bricks_left
        dec     (hl)
        ld      de,plus1+1
        ld      hl,score+1
        call    ADD_BCD

        ld      a,(temp_x)
        rra
        rra
        rra
        and     31
        ld      hl,working_map+(10*MAP_WIDTH)
        call    ADD_HL_A
        ld      a,(hl)
        or      a
        jr      nz,hit_c
        ld      a,(game_timer)
        and     7
        call    z,deploy_bonus
        jr      hit_c

hit_two:
        dec     (hl)
        call    get_block_coords
        ld      hl,tiles+24
        call    drw_block
hit_c:  call    get_block_coords
        ld      hl,tiles+12
        call    drw_block
        ld      a,1
        add     a,a
        ret

get_block_coords:
        ld      a,(temp_x)
        and     $f8
        ld      d,a
        ld      a,(temp_y)
        ld      e,a
        add     a,a
        add     a,e
        add     a,a             ; A = 6 * Y coordinate
        ld      e,a
        ret
