;##################################################################
;
;   TesbWan - panel display
;
;   Programmed by Patrick Davidson (pad@calc.org)
;        
;   This program is in the public domain.  There is no warranty.
;
;   This file was last updated August 24, 2002.
;
;##################################################################

;############## Show information panel

show_panel:
        ld      hl,(gfx_buffer)
        ld      de,$07e
        add     hl,de
        ex      de,hl
        ld      hl,score
        call    display_number_bcd

        ld      hl,(gfx_buffer)
        ld      de,$37e
        add     hl,de
        ex      de,hl
        ld      hl,number
        ld      a,(num_cycles)
        ld      (number+1),a
        jp      display_number_bcd

show_level:
        ld      hl,(gfx_buffer)
        ld      de,$17e
        add     hl,de
        ex      de,hl
        ld      hl,level
        jp      display_number_bcd

show_lives:
        ld      hl,(gfx_buffer)
        ld      de,$27e
        add     hl,de
        ex      de,hl
        ld      hl,lives
        jp      display_number_bcd

number: .db     0,0
minus1: .db     $99,$99
plus1:  .db     $00,$01

;############## Initialize infomration panel

init_panel:
        call    _clrLCD
        ld      hl,$006c
        ld      (_penCol),hl
        ld      hl,score_msg
        call    _vputs

        ld      hl,$106c
        ld      (_penCol),hl
        ld      hl,level_msg
        call    _vputs

        ld      hl,$206c
        ld      (_penCol),hl
        ld      hl,lives_msg
        call    _vputs

        ld      hl,$306c
        ld      (_penCol),hl
        ld      hl,magic_msg
        call    _vputs

        call    rrall
        call    rrall
        call    rrall
        call    rrall
        call    rrall

        ld      hl,$ffff
        ld      de,$fbff
        ld      bc,$c00
        lddr

        ld      hl,$ffff
        ld      de,back_buffer+$7ff
        ld      bc,$800
        lddr
        ret

rrall:  ld      hl,$fc00
rralll: rr      (hl)
        inc     hl
        ld      b,h
        djnz    rralll
        ret

lives_msg:
        .db     "lives",0
level_msg:
        .db     "|eve|",0
score_msg:
        .db     "pts.",0
magic_msg:
        .db     "ints",0

;############## Add two six-digit packed BCD numbers
;
; HL -> destination + 1
; DE -> source + 1
;
; Returns with the C flag 0 if no carry out, 1 if carry out.

ADD_BCD:
        ld      b,2
        and     a
loop_bcd:
        ld      a,(de)
        adc     a,(hl)
        daa
        ld      (hl),a
        dec     hl
        dec     de
        djnz    loop_bcd
        ret

;############## Compares BCD numbers
;
; HL ->number to subtract from other
; DE ->number
;
; Returns with flags indicating comparion results.  Changes A, B, DE, HL.

CP_BCD: ld      b,2
cmpl:   ld      a,(de)
        cp      (hl)
        ret     nz
        inc     de
        inc     hl
        djnz    cmpl
        ret

;############## Display hex/BCD numbers in special text (4 digits)
;
; HL -> location of number in memory
; DE -> graphics buffer address (upper-left corner of number)

display_number_bcd:
        ld      b,2
main_bcd_display:
        ld      a,(hl)
        push    de
        push    hl
        push    bc
        call    display_a_bcd
        pop     bc
        pop     hl
        pop     de
        inc     de
        inc     hl
        djnz    main_bcd_display
        ret

display_a_bcd:
        push    af                      ; Save character
        and     15                      ; A = low digit
        ld      c,a
        add     a,a
        add     a,a                   
        add     a,c
        ld      c,a
        ld      b,0
        ld      hl,digit_images
        add     hl,bc                   ; HL -> start of image

        ex      de,hl                   ; DE -> char image, HL -> screen
        push    hl                      ; Save screen address
        ld      c,16
        ld      a,5
loop_copy_digit:
        push    af
        ld      a,(de)
        ld      (hl),a
        pop     af
        inc     de
        add     hl,bc
        dec     a
        jr      nz,loop_copy_digit
        pop     de
                                        ; Restore screen address in DE
        pop     af
        rrca
        rrca
        rrca
        rrca
        and     15
        ld      c,a
        add     a,a
        add     a,a                    
        add     a,c
        ld      c,a
        ld      hl,digit_images
        add     hl,bc                   ; HL -> start of image

        ex      de,hl                   ; DE -> char image, HL -> screen
        ld      c,16
        ld      a,5
loop_or_digit:
        push    af
        ld      a,(de)
        add     a,a
        add     a,a
        add     a,a
        add     a,a
        or      (hl)
        ld      (hl),a
        pop     af
        inc     de
        add     hl,bc
        dec     a
        jr      nz,loop_or_digit
        ret
        
digit_images:
        .db     %00000010
        .db     %00000101
        .db     %00000101
        .db     %00000101
        .db     %00000010

        .db     %00000010
        .db     %00000010
        .db     %00000010
        .db     %00000010
        .db     %00000010

        .db     %00000010
        .db     %00000101
        .db     %00000001
        .db     %00000010
        .db     %00000111

        .db     %00000110
        .db     %00000001
        .db     %00000110
        .db     %00000001
        .db     %00000110

        .db     %00000101
        .db     %00000101
        .db     %00000111
        .db     %00000001
        .db     %00000001

        .db     %00000111
        .db     %00000100
        .db     %00000110
        .db     %00000001
        .db     %00000110

        .db     %00000011
        .db     %00000100
        .db     %00000110
        .db     %00000101
        .db     %00000010

        .db     %00000111
        .db     %00000001
        .db     %00000001
        .db     %00000010
        .db     %00000010

        .db     %00000010
        .db     %00000101
        .db     %00000010
        .db     %00000101
        .db     %00000010

        .db     %00000010
        .db     %00000101
        .db     %00000011
        .db     %00000001
        .db     %00000110
