;##################################################################
;
;   TesBwan for TI-86 (Initialization, Load/Save)
;
;   Programmed by Patrick Davidson (pad@calc.org)
;        
;   This program is in the public domain.  There is no warranty.
;
;   This file was last updated August 24, 2002.
;
;##################################################################

#define __TI86__

working_map     =$B000
back_buffer     =$9000

interrupt_entry =$CBCB
interrupt_byte  =$CB
interrupt_table =$CA00
interrupt_reg   =$CA

#include "ti-86.i"
#include "keys.i"
#include "tesbwan.i"

;############## Program header

        .org    _asm_exec_ram

        nop
        jp      startup
        .dw     0
        .dw     title

;############## Permanent program data

data_start:
#include "vars.i"
#include "scores.i"
data_end:

;############## Main program text

title:  .db     "TesBwan 0.3 by Patrick D",0

;############## Essential TI-86 initialization

startup:
        call    _flushAllMenus
        call    _runindicoff
        ld      (iy+13),0

        di

;############## Move the stack

        ld      hl,$FC00-400
        ld      de,_plotSScreen+$400-400
        ld      bc,400
        ldir
        ld      (oldsp+1),sp
        ld      hl,_plotSScreen+$400-$FC00
        add     hl,sp
        ld      sp,hl

;############## Run the program

        ld      (exit_program+1),sp
resume_program:
        call    main_code

;############## Exit routines

exit_program:
        ld      sp,0
        di
        im      1
        ld      a,$3c
        out     (0),a

        ld      a,(exit_status)
        or      a
        jr      z,just_exit

        dec     a
        call    z,enter_table

        call    show_table
        call    wait_key

        ld      a,(exit_status)
        cp      2
        jr      z,resume_program

just_exit:
        ld      hl,textShadow
        ld      de,textShadow+1
        ld      (hl),' '
        ld      bc,167
        ldir
        call    _clrLCD
        ld      hl,0
        ld      (_curRow),hl
        ld      (iy+13),6

;############## Save data

        ld      hl,name
        call    _MOV10TOOP1
        call    _FINDSYM                ; BDE -> start of program
        ld      hl,data_start+4-_asm_exec_ram
        ld      a,b
        add     hl,de
        adc     a,0                     ; AHL -> perm storage in variable
        call    _SET_ABS_DEST_ADDR
        ld      a,0
        ld      hl,data_start
        call    _SET_ABS_SRC_ADDR
        ld      a,0
        ld      hl,data_end-data_start
        call    _SET_MM_NUM_BYTES
        call    _mm_ldir             

;############## Restore stack to original location

oldsp:  ld      sp,0
        ld      de,$FC00-400
        ld      hl,_plotSScreen+$400-400
        ld      bc,400
        ldir

        ret

#include "lib.asm"
#include "gfx.asm"
#include "info.asm"
#include "main.asm"
#include "drwspr.asm"
#include "drwblock.asm"
#include "paddle.asm"
#include "ball.asm"
#include "bricks.asm"
#include "levels.asm"
#include "bonuses.asm"
#include "title.asm"
#include "scores.asm"

tiles:
#include "tiles.inc"

name:   .db     $12,7,"tesbwan"

temp_area:

        .end
